'
' A sqlite3 demo program
' Examples found at http://zetcode.com/db/sqlitec/
' Create a SQL database
' Add a table
' Use SELECT to retrieve all data from table
' Query database using a prameterized query
' Retrieve metadata about the database using PRAGMA command
' Retrieve list of all tables in database
'
VERSION "0.0001"
CONSOLE
'
IMPORT "xst"				' Standard library : required by most programs
IMPORT "xsx"				' Extended standard library
IMPORT "sqlite3"

DECLARE FUNCTION Entry ()
DECLARE CFUNCTION callback (void, argc, rowData, colName)

$$DIVIDER$ = "====================================="

FUNCTION Entry ()

  rc = sqlite3_open(&"test.db", &db)

  IF (rc != $$SQLITE_OK) THEN
    PRINT "Cannot open database: "; CSTRING$(sqlite3_errmsg(db))
    GOTO end
  END IF

  sql$ = "DROP TABLE IF EXISTS Cars; CREATE TABLE Cars(Id INT, Name TEXT, Price INT); INSERT INTO Cars VALUES(1, 'Audi', 52642);" + _
	       "INSERT INTO Cars VALUES(2, 'Mercedes', 57127); INSERT INTO Cars VALUES(3, 'Skoda', 9000);" + _
				 "INSERT INTO Cars VALUES(4, 'Volvo', 29000); INSERT INTO Cars VALUES(5, 'Bentley', 350000);" + _
				 "INSERT INTO Cars VALUES(6, 'Citroen', 21000); INSERT INTO Cars VALUES(7, 'Hummer', 41400);" + _
         "INSERT INTO Cars VALUES(8, 'Volkswagen', 21600);"

' sqlite3_exec() wraps sqlite3_prepare_v2(), sqlite3_step(), and sqlite3_finalize() together which allows
' an app to run multiple statements of SQL without using extra code

  rc = sqlite3_exec(db, &sql$, 0, 0, &err_msg)

  IF (rc != $$SQLITE_OK ) THEN
    PRINT "SQL error: "; CSTRING$(err_msg)
    sqlite3_free(err_msg)
    GOTO end
	ELSE
		PRINT "Table Cars created successfully"
  END IF

' get id of last inserted row

	last_id = sqlite3_last_insert_rowid(db)
  PRINT "The last ID of the inserted row is "; last_id

	PRINT $$DIVIDER$

' retrieve all rows of data from database using SELECT * FROM Cars
' the callback() function is invoked for each row coming out of the evaluated SQL statement

  sql$ = "SELECT * FROM Cars"

  rc = sqlite3_exec(db, &sql$, &callback(), 0, &err_msg)
  IF (rc != $$SQLITE_OK ) THEN
    PRINT "Failed to select data. SQL error: "; CSTRING$(err_msg)
    sqlite3_free(err_msg)
    GOTO end
  END IF

	PRINT $$DIVIDER$

' query database using a prameterized query or prepared statement
' ? is used as a placeholder for Id which is later replaced with a value

  sql$ = "SELECT Id, Name FROM Cars WHERE Id = ?"

  rc = sqlite3_prepare_v2(db, &sql$, -1, &res, 0)

' lookup name for Id = 3
  IF (rc == $$SQLITE_OK) THEN
    sqlite3_bind_int(res, 1, 3)
  ELSE
    PRINT "Failed to execute statement: "; CSTRING$(sqlite3_errmsg(db))
  END IF

' evaluate SQL statement
  step = sqlite3_step(res)

' get Id and car Name
  IF (step == $$SQLITE_ROW) THEN
    col0$ = CSTRING$(sqlite3_column_text(res, 0))
    col1$ = CSTRING$(sqlite3_column_text(res, 1))
		PRINT col0$; ":"; col1$
  END IF

  sqlite3_finalize(res)

	PRINT $$DIVIDER$

' retrieve some metadata about the database using PRAGMA command

 sql$ = "PRAGMA table_info(Cars)"

  rc = sqlite3_exec(db, &sql$, &callback(), 0, &err_msg)

  IF (rc != $$SQLITE_OK ) THEN
    PRINT "Failed to select data. SQL error: "; CSTRING$(err_msg)
    sqlite3_free(err_msg)
    GOTO end
  END IF

	PRINT $$DIVIDER$

' retrieve list of all tables in database

  sql$ = "SELECT name FROM sqlite_master WHERE type='table'"

  rc = sqlite3_exec(db, &sql$, &callback(), 0, &err_msg)

  IF (rc != $$SQLITE_OK ) THEN
    PRINT "Failed to select data. SQL error: "; CSTRING$(err_msg)
    sqlite3_free(err_msg)
    GOTO end
  END IF

end:
	sqlite3_close(db)

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

CFUNCTION callback (void, argc, pRowData, pColName)

	FOR i = 0 TO argc-1
		colNames$ = CSTRING$(XLONGAT(pColName + i*4))
		data$     = CSTRING$(XLONGAT(pRowData + i*4))
		IFZ data$ THEN data$ = "NULL"
		PRINT colNames$ + " = " + data$
	NEXT i

END FUNCTION
END PROGRAM