'
' A sqlite3 demo program
' C Examples found at https://www.tutorialspoint.com/sqlite/sqlite_c_cpp.htm
' Create a SQL database
' Add a table using CREATE TABLE
' Add records to table using INSERT INTO
' Display records using SELECT
' UPDATE a record
' DELETE a record
' Sort rocords using ORDER BY
VERSION "0.0001"
CONSOLE
'
IMPORT "xst"				' Standard library : required by most programs
IMPORT "xsx"				' Extended standard library
IMPORT "sqlite3"

DECLARE FUNCTION Entry ()
DECLARE CFUNCTION callback (void, argc, rowData, colName)

$$DIVIDER$ = "====================================="

FUNCTION Entry ()

  rc = sqlite3_open(&"test.db", &db)

  IF (rc != $$SQLITE_OK) THEN
    PRINT "Cannot open database: "; CSTRING$(sqlite3_errmsg(db))
    sqlite3_close(db)
    GOTO end
	ELSE
		PRINT "Opened database successfully."
  END IF

' Create SQL statement
  sql$ = "DROP TABLE IF EXISTS COMPANY; CREATE TABLE COMPANY(" + _
         "ID INT PRIMARY KEY     NOT NULL," + _
         "NAME           TEXT    NOT NULL," + _
         "AGE            INT     NOT NULL," + _
         "ADDRESS        CHAR(50)," + _
         "SALARY         REAL);"

' Execute SQL statement
  rc = sqlite3_exec(db, &sql$, 0, 0, &errMsg)
  IF (rc != $$SQLITE_OK ) THEN
		PRINT "SQL error: "; CSTRING$(errMsg)
    sqlite3_free(errMsg)
  ELSE
    PRINT "Table created successfully."
  END IF

' Create some records in the COMPANY table
' Create SQL statement
  sql$ = "INSERT INTO COMPANY (ID,NAME,AGE,ADDRESS,SALARY) " + _
         "VALUES (1, 'Paul', 32, 'California', 20000.00 ); " + _
         "INSERT INTO COMPANY (ID,NAME,AGE,ADDRESS,SALARY) " + _
         "VALUES (2, 'Allen', 25, 'Texas', 15000.00 ); " + _
         "INSERT INTO COMPANY (ID,NAME,AGE,ADDRESS,SALARY)" + _
         "VALUES (3, 'Teddy', 23, 'Norway', 20000.00 );" + _
         "INSERT INTO COMPANY (ID,NAME,AGE,ADDRESS,SALARY)" + _
         "VALUES (4, 'Mark', 25, 'Rich-Mond ', 65000.00 );"

' Execute SQL statement
  rc = sqlite3_exec(db, &sql$, &callback(), 0, &zErrMsg)
  IF (rc != $$SQLITE_OK ) THEN
    PRINT "SQL error: "; CSTRING$(zErrMsg)
    sqlite3_free(zErrMsg)
  ELSE
    PRINT "Records created successfully"
  END IF

	PRINT $$DIVIDER$

' Fetch and display records in COMPANY using SELECT statement
	sql$ = "SELECT * from COMPANY"
	rc = sqlite3_exec(db, &sql$, &callback(), data, &zErrMsg)
  IF (rc != $$SQLITE_OK ) THEN
    PRINT "SQL error: "; CSTRING$(zErrMsg)
    sqlite3_free(zErrMsg)
  ELSE
    PRINT "Operation done successfully."
  END IF

	PRINT $$DIVIDER$

' Use UPDATE statement to modify any record
  sql$ = "UPDATE COMPANY set SALARY = 25000.00 where ID=1; SELECT * from COMPANY"
	rc = sqlite3_exec(db, &sql$, &callback(), data, &zErrMsg)
  IF (rc != $$SQLITE_OK ) THEN
    PRINT "SQL error: "; CSTRING$(zErrMsg)
    sqlite3_free(zErrMsg)
  ELSE
    PRINT "Operation done successfully."
  END IF

	PRINT $$DIVIDER$

' Use DELETE statement to delete a record
  sql$ = "DELETE from COMPANY where ID=2; SELECT * from COMPANY"
	rc = sqlite3_exec(db, &sql$, &callback(), data, &zErrMsg)
  IF (rc != $$SQLITE_OK ) THEN
    PRINT "SQL error: "; CSTRING$(zErrMsg)
    sqlite3_free(zErrMsg)
  ELSE
    PRINT "Operation done successfully."
  END IF

	PRINT $$DIVIDER$

' Use ORDER BY statement to sort records by a column using ASC or DESC
  sql$ = "SELECT * from COMPANY ORDER BY AGE ASC"
	rc = sqlite3_exec(db, &sql$, &callback(), data, &zErrMsg)
  IF (rc != $$SQLITE_OK ) THEN
    PRINT "SQL error: "; CSTRING$(zErrMsg)
    sqlite3_free(zErrMsg)
  ELSE
    PRINT "Operation done successfully."
  END IF


  sqlite3_close(db)


end:

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

CFUNCTION callback (void, argc, pRowData, pColName)

	FOR i = 0 TO argc-1
		colNames$ = CSTRING$(XLONGAT(pColName + i*4))
		data$     = CSTRING$(XLONGAT(pRowData + i*4))
		IFZ data$ THEN data$ = "NULL"
		PRINT colNames$ + " = " + data$
	NEXT i

END FUNCTION
END PROGRAM