'
' Note that the callback function MyCallback() is declared as a CFUNCTION.

PROGRAM "demo7"
CONSOLE

IMPORT "xst"
IMPORT "xsx"
IMPORT "sqlite3"

DECLARE FUNCTION Main()
DECLARE CFUNCTION MyCallback (user, cols, pTextArray, pColNames)
DECLARE FUNCTION GetError (db)

FUNCTION Main()

	ret = sqlite3_libversion()
	PRINT "sqlite3 version: "; CSTRING$(ret)

	db$ = "test.db"

	'check if test.db exists
	IFZ XstFileExists(db$) THEN XstDeleteFile(db$)

	'create new database file
	ret = sqlite3_open (&db$, &dbHandle)
	IF ret THEN GetError(dbHandle)

	sql$ = "CREATE TABLE IF NOT EXISTS test (item varchar primary key, color text, price real);"
	ret = sqlite3_exec (dbHandle, &sql$, 0, 0, 0)
	PRINT "create table return code:"; ret
	IF ret THEN GetError(dbHandle)


	sql$ = "BEGIN TRANSACTION;INSERT INTO test VALUES ('apple1a','green',0.95);insert into test values ('apple2a','red',0.85);insert into test values ('apple3a','blue',0.85);insert into test values ('apple4a','red',0.85);insert into test values ('apple5a','green',0.85);insert into test values ('apple6a','red',0.85);insert into test values ('apple7a','blue',0.85);insert into test values ('apple8a','red',0.85);insert into test values ('apple9a','green',0.85);insert into test values ('tomato1a','green',0.65);insert into test values ('tomato2a','red',0.65);insert into test values ('tomato3a','red',0.65);insert into test values ('tomato4a','red',0.65);insert into test values ('tomato5a','red',0.65);insert into test values ('tomato6a','red',0.65);insert into test values ('tomato7a','red',0.65);insert into test values ('tomato8a','red',0.65);insert into test values ('tomato9a','red',0.65);insert into test values ('strawberry1a','red',0.15);insert into test values ('strawberry2a','red',0.25);insert into test values ('strawberry3a','red',0.35);insert into test values ('strawberry4a','red',0.45);insert into test values ('strawberry5a','red',0.55);insert into test values ('strawberry6a','red',0.65);insert into test values ('strawberry7a','red',0.75);insert into test values ('strawberry8a','red',0.85);insert into test values ('strawberry9a','red',0.95);insert into test values ('banana1a','yellow',1.15);insert into test values ('banana2a','yellow',1.25);insert into test values ('banana3a','yellow',1.35);insert into test values ('banana4a','yellow',1.45);insert into test values ('banana5a','yellow',1.55);insert into test values ('banana6a','yellow',1.65);insert into test values ('banana7a','yellow',1.75);insert into test values ('banana8a','yellow',1.85);insert into test values ('banana9a','yellow',1.95);insert into test values ('pineapple1','yellow',1.15);insert into test values ('pineapple2','yellow',1.25);insert into test values ('pineapple3','yellow',1.35);insert into test values ('pineapple4','yellow',1.45);insert into test values ('pineapple5','yellow',1.55);insert into test values ('pineapple6','yellow',1.65);insert into test values ('pineapple7','yellow',1.75);insert into test values ('pineapple8','yellow',1.85);insert into test values ('pineapple9','yellow',1.95);insert into test values ('beans1','green',0.35);insert into test values ('beans2','yellow',0.40);insert into test values ('beans3','red',2.00);insert into test values ('beans4','blue',1.40);insert into test values ('beans5','yellow',5.45);insert into test values ('beans6','yellow',0.40);insert into test values ('beans7','yellow',0.40);insert into test values ('beans8','yellow',0.40);insert into test values ('beans9','yellow',0.40);insert into test values ('dairy1','yellow',0.40);insert into test values ('dairy2','red',0.60);insert into test values ('dairy3','blue',0.80);insert into test values ('dairy4','green',0);COMMIT TRANSACTION;"
	' another way to insert multiple rows
	'sql$ = "INSERT INTO 'test' ('item', 'color', 'price') VALUES ('apple1a','green',0.95),('apple2a','red',0.85),('apple3a','blue',0.85),('apple4a','red',0.85),('apple5a','green',0.85),('apple6a','red',0.85),('apple7a','blue',0.85),('apple8a','red',0.85),('apple9a','green',0.85);"
	ret = sqlite3_exec (dbHandle, &sql$, 0, 0, 0)
	PRINT "insert row return code:"; ret
	IF ret THEN GetError(dbHandle)

	PRINT "===================================="
	sql$ = "SELECT * FROM test WHERE color = 'red';"
	count = 0
	ret = sqlite3_exec (dbHandle, &sql$, &MyCallback(), &count, 0)
	PRINT "===================================="
	PRINT STRING$(count) + " items found which are red"
	PRINT "select return code:"; ret
	IF ret THEN GetError(dbHandle)

	ret = sqlite3_close (dbHandle)
	PRINT "close return code:"; ret

	a$ = INLINE$ ("Press any key to quit > ")

END FUNCTION

CFUNCTION MyCallback (user, cols, pTextArray, pColNames)

STATIC count

	FOR i = 0 TO cols-1
		colNames$ = CSTRING$(XLONGAT(pColNames+i*4))
		text$     = CSTRING$(XLONGAT(pTextArray+i*4))
		PRINT colNames$ + " = " + text$
	NEXT i

	INC count
	XstCopyMemory (&count, user, 4)

END FUNCTION

FUNCTION GetError (db)

	DIM args[0]
	args[0] = db
	errcode = sqlite3_errcode (db)
	pMsg = sqlite3_errmsg (db)
	IF pMsg THEN errormsg$ = CSTRING$(pMsg)

	errorstr$ = errormsg$ + "\r\nerror : " + STRING (errcode)
	XstAbend (@errorstr$)

	sqlite3_close (db)

	QUIT (errcode)

END FUNCTION

END PROGRAM